package fr.soleil.docking;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.List;
import java.util.prefs.Preferences;

import javax.swing.Action;
import javax.swing.JComponent;

import fr.soleil.docking.action.SaveDefaultPerspertiveAction;
import fr.soleil.docking.exception.DockingException;
import fr.soleil.docking.perspective.IPerspective;
import fr.soleil.docking.perspective.IPerspectiveFactory;
import fr.soleil.docking.view.IViewFactory;
import fr.soleil.lib.project.ObjectUtils;

/**
 * A class that prepares docking. Obtain your RootWindow from this class.
 * 
 * @author HARDION
 * @author GIRARDOT
 */
public abstract class ADockingManager implements PropertyChangeListener {

    private boolean automaticallySavePerspective = true;

    public boolean isAutomaticallySavePerspective() {
        return automaticallySavePerspective;
    }

    public void setAutomaticallySavePerspective(boolean automaticallySavePerspective) {
        this.automaticallySavePerspective = automaticallySavePerspective;
    }

    protected IPerspectiveFactory perspectiveFactory = null;
    protected IViewFactory viewFactory = null;

    public ADockingManager(IViewFactory viewFactory, IPerspectiveFactory perspectiveFactory) {
        this.viewFactory = viewFactory;
        this.viewFactory.addPropertyChangeListener(this);
        this.perspectiveFactory = perspectiveFactory;
        this.perspectiveFactory.addPropertyChangeListener(this);
        this.getDockingArea();
        try {
            this.resetLayout();
        } catch (DockingException e) {
            e.printStackTrace();
        }
    }

    public List<Action> getActionList() {
        List<Action> result = null;
        if (viewFactory != null) {
            result = viewFactory.getActionList();
        }
        if (Boolean.getBoolean("DEBUG")) {
            result.add(new SaveDefaultPerspertiveAction(this));
        }
        return result;
    }

    /**
     * Returns this {@link ADockingManager}'s main docking area
     * 
     * @return A {@link JComponent}
     */
    public abstract JComponent getDockingArea();

    /**
     * Creates a new docking area
     * 
     * @param areaBackground The preferred area background. May be <code>null</code>, in which case this
     *            {@link ADockingManager} chooses the {@link Color} to use
     * 
     * @return A {@link JComponent}
     */
    public abstract JComponent createNewDockingArea(Color areaBackground);

    /**
     * Changes the background of a docking area
     * 
     * @param dockingArea The docking area
     * @param areaBackground The background to set
     */
    public abstract void setDockingAreaBeackground(JComponent dockingArea, Color areaBackground);

    /**
     * Returns the perspective factory
     * 
     * @return The perspective factory
     */
    public IPerspectiveFactory getPerspectiveFactory() {
        return perspectiveFactory;
    }

    /**
     * Returns the view factory
     * 
     * @return The view factory
     */
    public IViewFactory getViewFactory() {
        return viewFactory;
    }

    public void loadPreferences(Preferences prefs) throws DockingException {
        this.perspectiveFactory.loadPreferences(prefs);
        this.viewFactory.loadPreferences(prefs);
        this.loadPerspective(perspectiveFactory.getSelectedPerspective());
    }

    public void resetLayout() throws DockingException {
        loadPerspective(perspectiveFactory.getDefault());
    }

    /**
     * @param is
     */
    public abstract void loadPerspective(IPerspective perspective) throws DockingException;

    public void savePreferences(Preferences prefs) throws DockingException {
        // Save current Perspective
        this.savePerspective(this.perspectiveFactory.getSelectedPerspective());

        // Save Preferences of perspective Factory
        this.perspectiveFactory.savePreferences(prefs);

        // Save Preferences of Views
        this.viewFactory.savePreferences(prefs);

    }

    /**
     * @param perspective
     * @param out
     * @throws DockingException
     */
    protected abstract void savePerspective(IPerspective perspective) throws DockingException;

    public void saveDefault(File file) throws DockingException {
        this.savePerspective(this.perspectiveFactory.getSelectedPerspective());
        this.perspectiveFactory.saveSelected(file);
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt != null) {
            if (ObjectUtils.sameObject(perspectiveFactory, evt.getSource())) {
                if (IPerspectiveFactory.SELECTED_PERSPECTIVE.equals(evt.getPropertyName())) {
                    IPerspective oldd = (IPerspective) evt.getOldValue();
                    IPerspective neww = (IPerspective) evt.getNewValue();
                    try {
                        if (automaticallySavePerspective) {
                            this.savePerspective(oldd);
                        }
                        this.loadPerspective(neww);
                    } catch (DockingException e) {
                        e.printStackTrace();
                    }
                }
            } else if (ObjectUtils.sameObject(viewFactory, evt.getSource())) {
                if (IViewFactory.VIEWS.equals(evt.getPropertyName())) {
                    this.updateViews(evt);
                }
            }
        }
    }

    protected abstract void updateViews(PropertyChangeEvent evt);

    public void saveSelectedPerspective() throws DockingException {
        this.savePerspective(this.perspectiveFactory.getSelectedPerspective());
    }

    /**
     * Enable the docking or not If the docking is disable, the inherit class must disable all
     * functionality of the docking. I must be comparable to a JPanel composed by JtabbedPane,
     * JSplitPane etc...
     * 
     * @param enabledDocking
     */
    public abstract void setEnabledDocking(boolean enabledDocking);

    /**
     * Get the Enabled Docking property
     * 
     * @return true if the docking is enabled
     */
    public abstract boolean isEnabledDocking();

}
